import json
from pathlib import Path
from typing import Any, Dict

from ucsschool.kelvin.client import Session as _Session

kelvin_session_kwargs: Dict[str, Any] = {}


class Session(_Session):

    async def request(self, *args, **kwargs) -> Any:
        # Need to overwrite because the timeout can't be set in the session (Bug #54462)
        kwargs.update({"timeout": 4 * 60})  # The OpenAPI client uses 5 * 60
        return await super().request(*args, **kwargs)


_kelvin_session: Session = None


def load_kelvin_session_kwargs(path: Path) -> Dict[str, str]:
    try:
        with path.open() as settings_file:
            json_settings = json.load(settings_file)
        # only return required settings
        return {
            "username": json_settings["username"],
            "password": json_settings["password"],
            "host": json_settings["host"],
            "verify": json_settings["verify_ssl"],
        }
    except (EnvironmentError, KeyError, ValueError) as exc:
        raise ValueError(f"Missing, incomplete or malformed settings file '{path!s}': {exc!s}") from exc


async def kelvin_session() -> Session:
    # this function must be async, or Session.__init__() will fail because there is not yet an event loop
    global _kelvin_session
    if not _kelvin_session:
        _kelvin_session = Session(**kelvin_session_kwargs)
        _kelvin_session.open()
    return _kelvin_session
