from fastapi import APIRouter, Depends, HTTPException, status

from id_broker_common.kelvin import kelvin_session
from self_disclosure_plugin.dependencies import (
    extract_members,
    get_single_user_by_broker_id,
    schoolclass_getter,
    token_dep,
)
from self_disclosure_plugin.models import ClassMembers
from ucsschool.apis.models import OAuth2Token
from ucsschool.apis.opa import OPAClient, opa_instance
from ucsschool.kelvin.client import SchoolClass, Session, User as KelvinUser

router = APIRouter(tags=["groups"])


@router.get("/{group_id}/users", response_model=ClassMembers)
async def group_users(
    group_id: str,  # DEMOSCHOOL-Democlass
    token: OAuth2Token = Depends(token_dep),
    policy_instance: OPAClient = Depends(opa_instance),
    session: Session = Depends(kelvin_session),
):
    try:
        user: KelvinUser = await get_single_user_by_broker_id(token.sub, session)
    except HTTPException:
        raise HTTPException(status_code=status.HTTP_401_UNAUTHORIZED)
    school, classname = group_id.split("-", 1)
    school_class: SchoolClass = await schoolclass_getter(school, classname, session)
    await policy_instance.check_policy_true_or_raise(
        "self_disclosure_plugin/groups_members",
        {"user": user.as_dict(), "group": school_class.as_dict()},
    )
    classmembers = await extract_members(school_class, session)

    return classmembers
